@@ -1,7 +1,7 @@
 """Content items for Structured Report document instances."""
 import logging
 from copy import deepcopy
-from typing import cast, Optional, Sequence, Union
+from typing import cast, List, Optional, Sequence, Union
 
 import numpy as np
 from pydicom._storage_sopclass_uids import (
@@ -11,6 +11,7 @@ from pydicom._storage_sopclass_uids import (
 from pydicom.dataset import Dataset
 from pydicom.sr.codedict import codes
 from pydicom.sr.coding import Code
+from highdicom.uid import UID
 from highdicom.sr.coding import CodedConcept
 from highdicom.sr.enum import (
     GraphicTypeValues,
@@ -21,6 +22,7 @@ from highdicom.sr.enum import (
 )
 from highdicom.sr.utils import find_content_items
 from highdicom.sr.value_types import (
+    ContentItem,
     CodeContentItem,
     CompositeContentItem,
     ContentSequence,
@@ -730,16 +732,16 @@ class ImageRegion3D(Scoord3DContentItem):
         return cast(ImageRegion3D, item)
 
 
-class VolumeSurface(Scoord3DContentItem):
+class VolumeSurface(ContentSequence):
 
-    """Content item representing a volume surface in the the three-dimensional
+    """Content sequence representing a volume surface in the the three-dimensional
     patient/slide coordinate system in millimeter unit.
     """
 
     def __init__(
         self,
         graphic_type: Union[GraphicTypeValues3D, str],
-        graphic_data: np.ndarray,
+        graphic_data: Union[np.ndarray, Sequence[np.ndarray]],
         frame_of_reference_uid: str,
         source_images: Optional[
             Sequence[SourceImageForSegmentation]
@@ -750,9 +752,29 @@ class VolumeSurface(Scoord3DContentItem):
         Parameters
         ----------
         graphic_type: Union[highdicom.sr.GraphicTypeValues3D, str]
-            name of the graphic type
-        graphic_data: Sequence[Sequence[int]]
-            ordered set of (row, column, frame) coordinate pairs
+            name of the graphic type. Permissible values are "ELLIPSOID",
+            "POINT", "ELLIPSE" or "POLYGON".
+        graphic_data: Union[np.ndarray, Sequence[np.ndarray]]
+            List of graphic data for elements of the volume surface. Each item
+            of the list should be a 2D numpy array representing the graphic
+            data for a single element of type ``graphic_type``.
+
+            If `graphic_type` is ``"ELLIPSOID"`` or ``"POINT"``, the volume
+            surface will consist of a single element that defines the entire
+            surface. Therefore, a single 2D NumPy array should be passed
+            as a list of length 1 or as a NumPy array directly.
+
+            If `graphic_type` is ``"ELLIPSE"`` or ``"POLYGON"``, the volume
+            surface will consist of two or more planar regions that together
+            define the surface. Therefore a list of two or more 2D NumPy
+            arrays should be passed.
+
+            Each 2D NumPy array should have dimension N x 3 where each row of
+            the array represents a coordinate in the 3D Frame of Reference. The
+            number, N, and meaning of the coordinates depends upon the value of
+            `graphic_type`. See :class:`highdicom.sr.GraphicTypeValues3D` for
+            details.
+
         frame_of_reference_uid: str
             unique identifier of the frame of reference within which the
             coordinates are defined
@@ -766,23 +788,51 @@ class VolumeSurface(Scoord3DContentItem):
         Either one or more source images or one source series must be provided.
 
         """  # noqa: E501
-        graphic_type = GraphicTypeValues3D(graphic_type)
-        if graphic_type != GraphicTypeValues3D.ELLIPSOID:
+        super().__init__()
+        self._graphic_type = GraphicTypeValues3D(graphic_type)
+
+        # Allow single 2D numpy array for backwards compatibility
+        if isinstance(graphic_data, np.ndarray) and graphic_data.ndim == 2:
+            graphic_data = [graphic_data]
+
+        if self._graphic_type in (
+            GraphicTypeValues3D.ELLIPSOID,
+            GraphicTypeValues3D.POINT
+        ):
+            if len(graphic_data) > 1:
+                raise ValueError(
+                    'If graphic type is "ELLIPSOID" or "POINT", graphic data '
+                    'should consist of a single item.'
+                )
+        elif self._graphic_type in (
+            GraphicTypeValues3D.ELLIPSE,
+            GraphicTypeValues3D.POLYGON
+        ):
+            if len(graphic_data) < 2:
+                raise ValueError(
+                    'If graphic type is "ELLIPSE" or "POLYGON", graphic data '
+                    'should consist of two or more items.'
+                )
+        else:
             raise ValueError(
-                'Graphic type for volume surface must be "ELLIPSOID".'
+                f'Graphic type "{self._graphic_type}" is not valid for volume '
+                'surfaces.'
+            )
+
+        for graphic_data_item in graphic_data:
+            self.append(
+                Scoord3DContentItem(
+                    name=CodedConcept(
+                        value='121231',
+                        meaning='Volume Surface',
+                        scheme_designator='DCM'
+                    ),
+                    frame_of_reference_uid=frame_of_reference_uid,
+                    graphic_type=self._graphic_type,
+                    graphic_data=graphic_data_item,
+                    relationship_type=RelationshipTypeValues.CONTAINS
+                )
             )
-        super().__init__(
-            name=CodedConcept(
-                value='121231',
-                meaning='Volume Surface',
-                scheme_designator='DCM'
-            ),
-            frame_of_reference_uid=frame_of_reference_uid,
-            graphic_type=graphic_type,
-            graphic_data=graphic_data,
-            relationship_type=RelationshipTypeValues.CONTAINS
-        )
-        self.ContentSequence = ContentSequence()
         if source_images is not None:
             for image in source_images:
                 if not isinstance(image, SourceImageForSegmentation):
@@ -790,14 +840,14 @@ class VolumeSurface(Scoord3DContentItem):
                         'Items of argument "source_image" must have type '
                         'SourceImageForSegmentation.'
                     )
-                self.ContentSequence.append(image)
+                self.append(image)
         elif source_series is not None:
             if not isinstance(source_series, SourceSeriesForSegmentation):
                 raise TypeError(
                     'Argument "source_series" must have type '
                     'SourceSeriesForSegmentation.'
                 )
-            self.ContentSequence.append(source_series)
+            self.append(source_series)
         else:
             raise ValueError(
                 'One of the following two arguments must be provided: '
@@ -805,27 +855,188 @@ class VolumeSurface(Scoord3DContentItem):
             )
 
     @classmethod
-    def from_dataset(cls, dataset: Dataset) -> 'VolumeSurface':
-        """Construct object from an existing dataset.
+    def from_sequence(
+        cls,
+        sequence: Sequence[Dataset]
+    ) -> 'VolumeSurface':
+        """Construct an object from an existing content sequence.
 
         Parameters
         ----------
-        dataset: pydicom.dataset.Dataset
-            Dataset representing an SR Content Item with value type SCOORD
+        sequence: Sequence[Dataset]
+            Sequence of datasets to be converted. This is expected to contain
+            one or more content items with concept name
+            'Volume Surface', and either a single
+            content item with concept name 'Source Series For Segmentation', or
+            1 or more content items with concept name
+            'Source Image For Segmentation'.
 
         Returns
         -------
         highdicom.sr.VolumeSurface
-            Constructed object
+            Constructed VolumeSurface object, containing copies
+            of the original content items.
 
         """
-        dataset_copy = deepcopy(dataset)
-        return cls._from_dataset(dataset_copy)
+        vol_surface_items: List[ContentItem] = []
+        source_image_items: List[ContentItem] = []
+        source_series_items: List[ContentItem] = []
+        for item in sequence:
+            name_item = item.ConceptNameCodeSequence[0]
+            name = Code(
+                value=name_item.CodeValue,
+                meaning=name_item.CodeMeaning,
+                scheme_designator=name_item.CodingSchemeDesignator
+            )
+            value_type = ValueTypeValues(item.ValueType)
+            rel_type = RelationshipTypeValues(item.RelationshipType)
 
-    @classmethod
-    def _from_dataset(cls, dataset: Dataset) -> 'VolumeSurface':
-        item = super()._from_dataset_base(dataset)
-        return cast(VolumeSurface, item)
+            if (
+                (name == codes.DCM.VolumeSurface) and
+                (value_type == ValueTypeValues.SCOORD3D) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                vol_surface_items.append(
+                    Scoord3DContentItem.from_dataset(item)
+                )
+            elif (
+                (name == codes.DCM.SourceImageForSegmentation) and
+                (value_type == ValueTypeValues.IMAGE) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                source_image_items.append(
+                    SourceImageForSegmentation.from_dataset(item)
+                )
+            elif (
+                (name == codes.DCM.SourceSeriesForSegmentation) and
+                (value_type == ValueTypeValues.UIDREF) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                source_series_items.append(
+                    SourceSeriesForSegmentation.from_dataset(item)
+                )
+
+        if len(vol_surface_items) == 0:
+            raise RuntimeError(
+                'Expected sequence to contain one or more content items with '
+                'concept name "Volume Surface". Found 0.'
+            )
+
+        for item in vol_surface_items:
+            if item.graphic_type != vol_surface_items[0].graphic_type:
+                raise RuntimeError(
+                    'Expected all VolumeSurface content items to have a common '
+                    'graphic type.'
+                )
+
+        if len(source_image_items) == 0 and len(source_series_items) == 0:
+            raise RuntimeError(
+                'Expected sequence to contain either at least one content item '
+                'with concept name "Source Image For Segmentation" or one '
+                'content item with concept name "Source Series For '
+                'Segmentation". Found neither.'
+            )
+        if len(source_image_items) > 0 and len(source_series_items) > 0:
+            raise RuntimeError(
+                'Sequence should not contain both content items '
+                'with concept name "Source Image For Segmentation" and a'
+                'content item with concept name "Source Series For '
+                'Segmentation".'
+            )
+
+        if len(source_image_items) > 0:
+            new_seq = ContentSequence(
+                vol_surface_items + source_image_items
+            )
+        else:
+            if len(source_series_items) > 1:
+                raise RuntimeError(
+                    'Sequence should contain at most one content item '
+                    'with concept name "Source Series For Segmentation". Found '
+                    f'{len(source_series_items)}.'
+                )
+            new_seq = ContentSequence(
+                vol_surface_items + source_series_items
+            )
+
+        new_seq.__class__ = cls
+        new_seq = cast(VolumeSurface, new_seq)
+        new_seq._graphic_type = vol_surface_items[0].graphic_type
+        return new_seq
+
+    @property
+    def graphic_type(self) -> GraphicTypeValues3D:
+        """highdicom.sr.GraphicTypeValues3D: Graphic type."""
+        return self._graphic_type
+
+    @property
+    def frame_of_reference_uid(self) -> UID:
+        """highdicom.UID: Frame of reference UID."""
+        return UID(self._graphic_data_items[0].frame_of_reference_uid)
+
+    def has_source_images(self) -> bool:
+        """Returns whether the object contains information about source images.
+
+        ReferencedSegment objects must either contain information about source
+        images or source series (and not both).
+
+        Returns
+        -------
+        bool:
+            True if the object contains information about source images. False
+            if the image contains information about the source series.
+
+        """
+        return len(self._lut[codes.DCM.SourceImageForSegmentation]) > 0
+
+    @property
+    def source_images_for_segmentation(
+        self
+    ) -> List[SourceImageForSegmentation]:
+        """List[highdicom.sr.SourceImageForSegmentation]:
+            Source images for the volume surface.
+        """
+        return self._lut[codes.DCM.SourceImageForSegmentation]
+
+    @property
+    def source_series_for_segmentation(
+        self
+    ) -> Optional[SourceSeriesForSegmentation]:
+        """Optional[highdicom.sr.SourceSeriesForSegmentation]:
+            Source series for the volume surface.
+        """
+        items = self._lut[codes.DCM.SourceSeriesForSegmentation]
+        if len(items) == 0:
+            return None
+        else:
+            return cast(SourceSeriesForSegmentation, items[0])
+
+    @property
+    def _graphic_data_items(self) -> List[Scoord3DContentItem]:
+        """List[highdicom.sr.Scoord3DContentItem]:
+            Graphic data elements that comprise the volume surface.
+        """
+        return cast(
+            List[Scoord3DContentItem],
+            self._lut[codes.DCM.VolumeSurface]
+        )
+
+    @property
+    def graphic_data(self) -> List[np.ndarray]:
+        """Union[np.ndarray, List[np.ndarray]]:
+            Graphic data arrays that comprise the volume surface.
+            For volume surfaces with graphic type ``"ELLIPSOID"``
+            or ``"POINT"``, this will be a single 2D Numpy array representing
+            the graphic data. Otherwise, it will be a list of 2D Numpy arrays
+            representing graphic data for each element of the volume surface.
+        """
+        if self.graphic_type in (
+            GraphicTypeValues3D.ELLIPSOID,
+            GraphicTypeValues3D.POINT
+        ):
+            return self._graphic_data_items[0].value
+        else:
+            return [item.value for item in self._graphic_data_items]
 
 
 class RealWorldValueMap(CompositeContentItem):
@@ -1065,6 +1276,74 @@ class ReferencedSegmentationFrame(ContentSequence):
             )
         self.append(source_image)
 
+    @classmethod
+    def from_sequence(
+        cls,
+        sequence: Sequence[Dataset]
+    ) -> 'ReferencedSegmentationFrame':
+        """Construct an object from items within an existing content sequence.
+
+        Parameters
+        ----------
+        sequence: Sequence[Dataset]
+            Sequence of datasets to be converted. This is expected to contain
+            content items with the following names:
+            "Referenced Segmentation Frame", "Source Image For Segmentation".
+            Any other other items will be ignored.
+
+        Returns
+        -------
+        highdicom.sr.ReferencedSegmentationFrame
+            Constructed ReferencedSegmentationFrame object, containing copies
+            of the relevant original content items.
+
+        """
+        seg_frame_items = []
+        source_image_items = []
+        for item in sequence:
+            name_item = item.ConceptNameCodeSequence[0]
+            name = Code(
+                value=name_item.CodeValue,
+                meaning=name_item.CodeMeaning,
+                scheme_designator=name_item.CodingSchemeDesignator
+            )
+            value_type = ValueTypeValues(item.ValueType)
+            rel_type = RelationshipTypeValues(item.RelationshipType)
+
+            if (
+                (name == codes.DCM.ReferencedSegmentationFrame) and
+                (value_type == ValueTypeValues.IMAGE) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                seg_frame_items.append(
+                    ImageContentItem.from_dataset(item)
+                )
+            elif (
+                (name == codes.DCM.SourceImageForSegmentation) and
+                (value_type == ValueTypeValues.IMAGE) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                source_image_items.append(
+                    SourceImageForSegmentation.from_dataset(item)
+                )
+
+        if len(seg_frame_items) != 1:
+            raise RuntimeError(
+                'Expected sequence to contain exactly one content item with '
+                'concept name "Referenced Segmentation Frame". Found '
+                f'{len(seg_frame_items)}.'
+            )
+        if len(source_image_items) != 1:
+            raise RuntimeError(
+                'Expected sequence to contain exactly one content item with '
+                'concept name "Source Image For Segmentation". Found '
+                f'{len(source_image_items)}.'
+            )
+
+        new_seq = ContentSequence([seg_frame_items[0], source_image_items[0]])
+        new_seq.__class__ = cls
+        return new_seq
+
     @classmethod
     def from_segmentation(
         cls,
@@ -1192,6 +1471,41 @@ class ReferencedSegmentationFrame(ContentSequence):
             source_image=source_image
         )
 
+    @property
+    def referenced_sop_class_uid(self) -> UID:
+        """highdicom.UID
+            referenced SOP Class UID
+        """
+        return self[0].referenced_sop_class_uid
+
+    @property
+    def referenced_sop_instance_uid(self) -> UID:
+        """highdicom.UID
+            referenced SOP Class UID
+        """
+        return self[0].referenced_sop_instance_uid
+
+    @property
+    def referenced_frame_numbers(self) -> Union[List[int], None]:
+        """Union[List[int], None]
+            referenced frame numbers
+        """
+        return self[0].referenced_frame_numbers
+
+    @property
+    def referenced_segment_numbers(self) -> Union[List[int], None]:
+        """Union[List[int], None]
+            referenced segment numbers
+        """
+        return self[0].referenced_segment_numbers
+
+    @property
+    def source_image_for_segmentation(self) -> SourceImageForSegmentation:
+        """highdicom.sr.SourceImageForSegmentation
+            Source image for the referenced segmentation
+        """
+        return self[1]
+
 
 class ReferencedSegment(ContentSequence):
 
@@ -1269,6 +1583,106 @@ class ReferencedSegment(ContentSequence):
                 '"source_images" or "source_series".'
             )
 
+    @classmethod
+    def from_sequence(
+        cls,
+        sequence: Sequence[Dataset]
+    ) -> 'ReferencedSegment':
+        """Construct an object from items within an existing content sequence.
+
+        Parameters
+        ----------
+        sequence: Sequence[Dataset]
+            Sequence of datasets to be converted. This is expected to contain
+            a content item with concept name "Referenced Segmentation Frame",
+            and either at least one content item with concept name
+            "Source Image For Segmentation" or a single content item with
+            concept name "Source Series For Segmentation".
+            Any other other items will be ignored.
+
+        Returns
+        -------
+        highdicom.sr.ReferencedSegment
+            Constructed ReferencedSegment object, containing copies
+            of the original content items.
+
+        """
+        seg_frame_items: List[ContentItem] = []
+        source_image_items: List[ContentItem] = []
+        source_series_items: List[ContentItem] = []
+        for item in sequence:
+            name_item = item.ConceptNameCodeSequence[0]
+            name = Code(
+                value=name_item.CodeValue,
+                meaning=name_item.CodeMeaning,
+                scheme_designator=name_item.CodingSchemeDesignator
+            )
+            value_type = ValueTypeValues(item.ValueType)
+            rel_type = RelationshipTypeValues(item.RelationshipType)
+
+            if (
+                (name == codes.DCM.ReferencedSegment) and
+                (value_type == ValueTypeValues.IMAGE) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                seg_frame_items.append(
+                    ImageContentItem.from_dataset(item)
+                )
+            elif (
+                (name == codes.DCM.SourceImageForSegmentation) and
+                (value_type == ValueTypeValues.IMAGE) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                source_image_items.append(
+                    SourceImageForSegmentation.from_dataset(item)
+                )
+            elif (
+                (name == codes.DCM.SourceSeriesForSegmentation) and
+                (value_type == ValueTypeValues.UIDREF) and
+                (rel_type == RelationshipTypeValues.CONTAINS)
+            ):
+                source_series_items.append(
+                    SourceSeriesForSegmentation.from_dataset(item)
+                )
+
+        if len(seg_frame_items) != 1:
+            raise RuntimeError(
+                'Expected sequence to contain exactly one content item with '
+                'concept name "Referenced Segment". Found 0.'
+            )
+        if len(source_image_items) == 0 and len(source_series_items) == 0:
+            raise RuntimeError(
+                'Expected sequence to contain either at least one content item '
+                'with concept name "Source Image For Segmentation" or one '
+                'content item with concept name "Source Series For '
+                'Segmentation". Found neither.'
+            )
+        if len(source_image_items) > 0 and len(source_series_items) > 0:
+            raise RuntimeError(
+                'Sequence should not contain both content items '
+                'with concept name "Source Image For Segmentation" and a'
+                'content item with concept name "Source Series For '
+                'Segmentation".'
+            )
+
+        if len(source_image_items) > 0:
+            new_seq = ContentSequence(
+                seg_frame_items + source_image_items
+            )
+        else:
+            if len(source_series_items) > 1:
+                raise RuntimeError(
+                    'Sequence should contain at most one content item '
+                    'with concept name "Source Series For Segmentation". Found '
+                    f'{len(source_series_items)}.'
+                )
+            new_seq = ContentSequence(
+                seg_frame_items + source_series_items
+            )
+
+        new_seq.__class__ = cls
+        return cast(ReferencedSegment, new_seq)
+
     @classmethod
     def from_segmentation(
         cls,
@@ -1441,3 +1855,71 @@ class ReferencedSegment(ContentSequence):
             source_images=source_images if source_images else None,
             source_series=source_series
         )
+
+    def has_source_images(self) -> bool:
+        """Returns whether the object contains information about source images.
+
+        ReferencedSegment objects must either contain information about source
+        images or source series (and not both).
+
+        Returns
+        -------
+        bool:
+            True if the object contains information about source images. False
+            if the image contains information about the source series.
+
+        """
+        return self[1].name == codes.DCM.SourceImageForSegmentation
+
+    @property
+    def referenced_sop_class_uid(self) -> UID:
+        """highdicom.UID
+            referenced SOP Class UID
+        """
+        return self[0].referenced_sop_class_uid
+
+    @property
+    def referenced_sop_instance_uid(self) -> UID:
+        """highdicom.UID
+            referenced SOP Class UID
+        """
+        return self[0].referenced_sop_instance_uid
+
+    @property
+    def referenced_frame_numbers(self) -> Union[List[int], None]:
+        """Union[List[int], None]
+            referenced frame numbers
+        """
+        return self[0].referenced_frame_numbers
+
+    @property
+    def referenced_segment_numbers(self) -> Union[List[int], None]:
+        """Union[List[int], None]
+            referenced segment numbers
+        """
+        return self[0].referenced_segment_numbers
+
+    @property
+    def source_images_for_segmentation(
+        self
+    ) -> List[SourceImageForSegmentation]:
+        """List[highdicom.sr.SourceImageForSegmentation]
+            Source images for the referenced segmentation
+        """
+        if self.has_source_images():
+            return list(self[1:])
+        else:
+            return []
+
+    @property
+    def source_series_for_segmentation(self) -> Union[
+        SourceSeriesForSegmentation,
+        None
+    ]:
+        """Union[highdicom.sr.SourceSeriesForSegmentation, None]
+            Source series for the referenced segmentation
+        """
+        if self.has_source_images():
+            return None
+        else:
+            return cast(SourceSeriesForSegmentation, self[1])
