@@ -23,12 +23,12 @@ def find_content_items(
     ----------
     dataset: pydicom.dataset.Dataset
         SR document instance
-    name: Union[highdicom.sr.CodedConcept, pydicom.sr.coding.Code], optional
+    name: Union[highdicom.sr.CodedConcept, pydicom.sr.coding.Code, None], optional
         Coded name that items should have
-    value_type: Union[highdicom.sr.ValueTypeValues, str], optional
+    value_type: Union[highdicom.sr.ValueTypeValues, str, None], optional
         Type of value that items should have
         (e.g. ``highdicom.sr.ValueTypeValues.CONTAINER``)
-    relationship_type: Union[highdicom.sr.RelationshipTypeValues, str], optional
+    relationship_type: Union[highdicom.sr.RelationshipTypeValues, str, None], optional
         Type of relationship that items should have with its parent
         (e.g. ``highdicom.sr.RelationshipTypeValues.CONTAINS``)
     recursive: bool, optional
@@ -45,20 +45,23 @@ def find_content_items(
     AttributeError
         When data set does not contain Content Sequence attribute.
 
-    """  # noqa
-    def has_name(item: ContentItem, name: Optional[str]) -> bool:
+    """  # noqa: E501
+    def has_name(
+        item: ContentItem,
+        name: Optional[Union[Code, CodedConcept]]
+    ) -> bool:
         if name is None:
             return True
         return item.name == name
 
     def has_value_type(
-            item: ContentItem,
-            value_type: Optional[Union[ValueTypeValues, str]]
+        item: ContentItem,
+        value_type: Optional[Union[ValueTypeValues, str]]
     ) -> bool:
         if value_type is None:
             return True
         value_type = ValueTypeValues(value_type)
-        return item.value_type == value_type.value
+        return item.value_type == value_type
 
     def has_relationship_type(
             item: ContentItem,
@@ -69,7 +72,7 @@ def find_content_items(
         if getattr(item, 'relationship_type', None) is None:
             return False
         relationship_type = RelationshipTypeValues(relationship_type)
-        return item.relationship_type == relationship_type.value
+        return item.relationship_type == relationship_type
 
     if not hasattr(dataset, 'ContentSequence'):
         raise AttributeError(
@@ -77,12 +80,12 @@ def find_content_items(
         )
 
     def search_tree(
-            node: Dataset,
-            name: Optional[Union[CodedConcept, Code]],
-            value_type: Optional[Union[ValueTypeValues, str]],
-            relationship_type: Optional[Union[RelationshipTypeValues, str]],
-            recursive: bool
-        ) -> List:
+        node: Dataset,
+        name: Optional[Union[CodedConcept, Code]],
+        value_type: Optional[Union[ValueTypeValues, str]],
+        relationship_type: Optional[Union[RelationshipTypeValues, str]],
+        recursive: bool
+    ) -> List:
         matched_content_items = []
         for i, content_item in enumerate(node.ContentSequence):
             name_code = content_item.ConceptNameCodeSequence[0]
@@ -139,12 +142,7 @@ def get_coded_name(item: Dataset) -> CodedConcept:
             'Dataset does not contain attribute "ConceptNameCodeSequence" and '
             'thus doesn\'t represent a SR Content Item.'
         )
-    return CodedConcept(
-        value=name.CodeValue,
-        scheme_designator=name.CodingSchemeDesignator,
-        meaning=name.CodeMeaning,
-        scheme_version=name.get('CodingSchemeVersion', None)
-    )
+    return CodedConcept.from_dataset(name)
 
 
 def get_coded_value(item: Dataset) -> CodedConcept:
@@ -168,9 +166,4 @@ def get_coded_value(item: Dataset) -> CodedConcept:
             'Dataset does not contain attribute "ConceptCodeSequence" and '
             'thus doesn\'t represent a SR Content Item of Value Type CODE.'
         )
-    return CodedConcept(
-        value=value.CodeValue,
-        scheme_designator=value.CodingSchemeDesignator,
-        meaning=value.CodeMeaning,
-        scheme_version=value.get('CodingSchemeVersion', None)
-    )
+    return CodedConcept.from_dataset(value)
